package cn.com.ty.lift.business.common;

import cn.com.ty.lift.business.framework.conf.SystemConfiguration;
import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.library.dao.entity.LiftBrand;
import cn.com.ty.lift.common.constants.RedisConstants;
import cn.com.ty.lift.common.model.AreaCode;
import cn.com.ty.lift.common.utils.AliyunOSS;
import cn.com.ty.lift.common.utils.OssEntity;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author bieao
 * @date 2019/12/19
 * @description 公共初始化数据
 */
@Slf4j
@AllArgsConstructor
@RestController
@RequestMapping("common")
public class CommonController {

    @Resource
    private RedisTemplate<String, Object> redisTemplate;

    private SystemConfiguration systemConfiguration;

    /**
     * @return RestResponse 省、市、区树形结构列表
     * @description 查询省、市、区树形结构列表
     * @date 2019/12/4 10:23 AM
     */
    @PostMapping("area/list")
    @SuppressWarnings("unchecked")
    public RestResponse areaList() {
        List<AreaCode> areaCodeList = (List<AreaCode>)redisTemplate.opsForValue().get(RedisConstants.RK_AREA_LIST);
        if (ObjectUtil.isEmpty(areaCodeList)) {
            return RestResponse.success();
        }
        return RestResponse.success(areaCodeList, MessageUtils.get("msg.query.success"));
    }

    /**
     * @return RestResponse 电梯品牌列表
     * @description 查询电梯品牌列表
     * @date 2019/12/10 10:23 AM
     */
    @PostMapping("brand/list")
    @SuppressWarnings(value = "unchecked")
    public RestResponse list(){
        List<LiftBrand> liftBrandList = (List<LiftBrand>) redisTemplate.opsForValue().get(RedisConstants.RK_LIFT_BRAND_LIST);
        if (ObjectUtil.isEmpty(liftBrandList)) {
            return RestResponse.success();
        }
        return RestResponse.success(liftBrandList, MessageUtils.get("msg.query.success"));
    }

    /**
     * 上传单个图片到aliyun OSS，返回图片的地址
     * @param file
     * @return
     */
    @PostMapping("upload/image")
    public RestResponse uploadImage(@RequestParam("file") MultipartFile file) {
        return upload(file,Values.Upload.pics,Values.Upload.maxSizePic);
    }

    /**
     * 上传多个图片，返回图片地址集合list
     * @param files
     * @return
     */
    @PostMapping("upload/images")
    public RestResponse uploadImage(@RequestParam("file") MultipartFile[] files) {
        return uploads(files,Values.Upload.pics,Values.Upload.maxSizePic);
    }

    /**
     * 上传单个文件，返回文件地址
     * @param file
     * @return
     */
    @PostMapping("upload/file")
    public RestResponse uploadFile(@RequestParam("file") MultipartFile file) {
        return upload(file,Values.Upload.files,Values.Upload.maxSizeFile);
    }

    /**
     * 上传多个文件，返回文件地址集合list
     * @param files
     * @return
     */
    @PostMapping("upload/files")
    public RestResponse uploadFile(@RequestParam("file") MultipartFile[] files) {
        return uploads(files,Values.Upload.files,Values.Upload.maxSizeFile);
    }

    /**
     * 上传单个视频，返回视频地址
     * @param file
     * @return
     */
    @PostMapping("upload/video")
    public RestResponse uploadVideo(@RequestParam("file") MultipartFile file) {
        return upload(file,Values.Upload.videos,Values.Upload.maxSizeVideo);
    }

    /**
     * 上传多个视频，返回视频地址集合list
     * @param files
     * @return
     */
    @PostMapping("upload/videos")
    public RestResponse uploadVideo(@RequestParam("file") MultipartFile[] files) {
        return uploads(files,Values.Upload.videos,Values.Upload.maxSizeVideo);
    }

    private RestResponse uploads(MultipartFile[] files,String[] types,long maxSize){
        try {
            Judge.notNull(files,Values.Upload.missingFileData);
            List<OssEntity> ossEntities = new ArrayList<>();
            for (MultipartFile file : files) {
                Judge.notTrue(null == file || file.isEmpty(),Values.Upload.missingFileData);
                Judge.notTrue(file.getSize() > maxSize,"文件大小不超过" + (maxSize >> 20) + "M");
                // 获取文件名，带后缀
                String originalFilename = file.getOriginalFilename();
                log.info("上传文件，原文件名：" + originalFilename);
                // 获取文件的后缀格式
                String fileSuffix = originalFilename.substring(originalFilename.lastIndexOf(".")).toLowerCase();
                Judge.isTrue(StrUtil.equalsAny(fileSuffix,types),"文件格式不支持, 暂时只支持" + types.toString());
                String filename = StrUtil.format("{}/{}{}",DateUtil.format(DateUtil.date(),"yyyy/MM/dd"),IdWorker.getIdStr(),fileSuffix);
                log.info("上传文件，新文件名：" + filename);
                OssEntity entity = new OssEntity();
                entity.setObjectName(filename);
                entity.setContent(file.getBytes());
                ossEntities.add(entity);
            }
            Judge.notNull(ossEntities);
            AliyunOSS aliyunOSS = systemConfiguration.build();
            Map<String, String> urls = aliyunOSS.putObjects(systemConfiguration.getBucketName(), ossEntities);
            Judge.notNull(urls);
            return Judge.result(urls.values());
        }catch (Exception e){
            log.error("上传文件异常:{}",e);
            return RestResponse.fail(Values.Upload.fileFail);
        }
    }

    private RestResponse upload(MultipartFile file,String[] types,long maxSize){
        try {
            Judge.notTrue(null == file || file.isEmpty(),Values.Upload.missingFileData);
            Judge.notTrue(file.getSize() > maxSize,"文件大小不超过" + (maxSize >> 20) + "M");
            // 获取文件名，带后缀
            String originalFilename = file.getOriginalFilename();
            log.info("上传文件，原文件名：" + originalFilename);
            // 获取文件的后缀格式
            String fileSuffix = originalFilename.substring(originalFilename.lastIndexOf(".")).toLowerCase();
            Judge.isTrue(StrUtil.equalsAny(fileSuffix,types),"文件格式不支持, 暂时只支持" + types.toString());
            String filename = StrUtil.format("{}/{}{}",DateUtil.format(DateUtil.date(),"yyyy/MM/dd"),IdWorker.getIdStr(),fileSuffix);
            log.info("上传文件，新文件名：" + filename);
            AliyunOSS aliyunOSS = systemConfiguration.build();
            String url = aliyunOSS.putObject(systemConfiguration.getBucketName(), filename, file.getBytes());
            log.info("上传文件，文件URL: {}",url);
            return Judge.result(url);
        }catch (Exception e){
            log.error("上传文件异常:{}",e);
            return RestResponse.fail(Values.Upload.fileFail);
        }
    }
}
