package cn.com.ty.lift.business.common;

import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;

import java.lang.reflect.Field;
import java.util.Arrays;
import java.util.List;

/**
 * @author wcz
 * @Description: 条件判断，如果符合条件，返回异常
 * @date 2020/1/9 8:46
 */
public interface Judge {

    String INVALID_IDENTIFIER = "标识ID无效";
    String MISSING_INCORRECT_DATA = "缺少数据或数据有误";
    String INVALID_DATA_CONDITIONS = "数据无效或条件不成立";

    /**
     * 检查id类型值，如果为null或者<= 0 ，直接抛出异常，返回结果
     * @param id
     */
    static void id(Long id) {
        if(null == id || id <= 0){
            throw new JudgeException(INVALID_IDENTIFIER);
        }
    }

    /**
     * 判断多个id有效，ids为空，或者出现id <= 0，抛出异常
     * @param ids
     */
    static void ids(Long... ids){
        notNull(ids,INVALID_IDENTIFIER);
        isTrue(ids.length > 0,INVALID_IDENTIFIER);
        List<Long> list = Arrays.asList(ids);
        list.forEach(id -> {
            id(id);
        });
    }

    /**
     * 判断对象object是否为空，如果不为空,直接抛出异常，返回message
     * @param object
     * @param message
     */
    static void isNull(Object object, String message){
        if(ObjectUtil.isNotEmpty(object)){
            throw new JudgeException(message);
        }
    }

    /**
     * 判断对象object是否为空，如果不为空,直接抛出异常，返回默认消息
     * @param object
     */
    static void isNull(Object object){
        isNull(object,MISSING_INCORRECT_DATA);
    }
    /**
     * 判断对象object是否为空，如果为真,直接抛出异常，返回message
     * @param object
     * @param message
     */
    static void notNull(Object object, String message){
        if(ObjectUtil.isEmpty(object)){
            throw new JudgeException(message);
        }
    }

    /**
     * 判断对象object是否为空，如果为真,直接抛出异常，返回默认消息
     * @param object
     */
    static void notNull(Object object){
        notNull(object,MISSING_INCORRECT_DATA);
    }

    /**
     * 判断表达式是否为真，如果为假，直接抛出异常，返回message
     * @param expression
     * @param message
     */
    static void isTrue(boolean expression, String message){
        if (!expression) {
            throw new JudgeException(message);
        }
    }

    /**
     * 判断表达式是否为真，如果为假，直接抛出异常，返回
     * @param expression
     */
    static void isTrue(boolean expression){
        isTrue(expression,INVALID_DATA_CONDITIONS);
    }

    /**
     * 判断表达式是否为真，如果为真，直接抛出异常，返回message
     * @param expression
     * @param message
     */
    static void notTrue(boolean expression, String message){
        if (expression) {
            throw new JudgeException(message);
        }
    }

    /**
     * 判断表达式是否为真，如果为真，直接抛出异常，返回
     * @param expression
     */
    static void notTrue(boolean expression){
        notTrue(expression,INVALID_DATA_CONDITIONS);
    }


    /**
     * 判断object,如果为空，返回fail，如果不为空，返回success(object);
     * @param object
     * @return
     */
    static RestResponse result(Object object){
        if(object instanceof Boolean){
            return (Boolean)object ? RestResponse.success(object) : RestResponse.fail();
        }
        if(object instanceof IPage){
            IPage page = (IPage) object;
            return page.getTotal() > 0 ? RestResponse.success(object) : RestResponse.success();
        }
        if(object instanceof CountPage){
            CountPage page = (CountPage) object;
            return page.getCount() > 0 ? RestResponse.success(object) : RestResponse.success();
        }
        return ObjectUtil.isNotEmpty(object) ? RestResponse.success(object) : RestResponse.success();
    }

    /**
     * 根据属性名检查object中是否有效值
     * @param object
     * @param fields
     */
    static void checkParameter(Object object,String... fields){
        notNull(object);
        Class<?> objectClass = object.getClass();
        notNull(objectClass);
        try {
            if(ArrayUtil.isNotEmpty(fields)){
                for (String field : fields) {
                    Field declaredField = objectClass.getDeclaredField(field);
                    declaredField.setAccessible(true);
                    Object value = declaredField.get(object);
                    if("id".equals(field) || field.endsWith("Id")){
                        id((Long) value);
                    }
                    notNull(value,field + ": " + MISSING_INCORRECT_DATA);
                }
            }
        }catch (Exception e){
            throw new JudgeException(e.getMessage());
        }
    }
}
