package cn.com.ty.lift.business.emergency.service;

import cn.com.ty.lift.business.common.Judge;
import cn.com.ty.lift.business.common.Values;
import cn.com.ty.lift.business.emergency.dto.RepairRequest;
import cn.com.ty.lift.business.emergency.dto.RepairResponse;
import cn.com.ty.lift.business.emergency.entity.EmergencyRepair;
import cn.com.ty.lift.business.emergency.entity.ErCostItem;
import cn.com.ty.lift.business.emergency.entity.ErRecordImg;
import cn.com.ty.lift.business.emergency.mapper.EmergencyRepairMapper;
import cn.com.ty.lift.business.library.dao.entity.PlatformCompanyLiftRelevance;
import cn.com.ty.lift.business.library.service.PlatformCompanyLiftRelevanceService;
import cn.com.ty.lift.common.constants.CommonEnum;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * 临修记录 服务实现类
 * </p>
 *
 * @author wcz
 * @since 2019-12-04
 */
@Service
@AllArgsConstructor
public class EmergencyRepairService extends ServiceImpl<EmergencyRepairMapper, EmergencyRepair> {

    private PlatformCompanyLiftRelevanceService platformCompanyLiftRelevanceService;
    private ErRecordImgService erRecordImgService;
    private ErCostItemService erCostItemService;

    /**
     * 分页查询急修中的急修记录
     * @param request
     * @return
     */
    public IPage<RepairResponse> pageByRepairing(RepairRequest request) {
        long pageSize = request.getPageSize();
        long pageNum = request.getPageNum();
        IPage<RepairResponse>  page = new Page<>(pageNum,pageSize);
        return baseMapper.pageByRepairing(page,request);
    }

    /**
     * 根据条件分页查询急修记录
     * @param request
     * @return
     */
    public IPage<RepairResponse> pageByCondition(RepairRequest request) {
        long pageNum = request.getPageNum();
        long pageSize = request.getPageSize();
        IPage<RepairResponse> page = new Page<>(pageNum,pageSize);
        return baseMapper.pageByCondition(page,request);
    }

    /**
     * 根据条件统计急修记录总条数
     * @param request
     * @return
     */
    public long countByCondition(RepairRequest request) {
        return baseMapper.countByCondition(request);
    }

    /**
     * 根据id查询急修记录，带关联信息
     * @param request
     * @return
     */
    public RepairResponse selectByIdWithInfo(RepairRequest request) {
        return baseMapper.selectByIdWithInfo(request);
    }

    /**
     * 根据用户id查询关联的急修
     * @param request
     * @return
     */
    public IPage<RepairResponse> pageByUser(RepairRequest request) {
        long pageNum = request.getPageNum();
        long pageSize = request.getPageSize();
        IPage<RepairResponse> page = new Page<>(pageNum,pageSize);
        return baseMapper.pageByUser(page,request);
    }

    /**
     * 急修接单，修改急修为修理中，修改电梯的业务状态为急修中
     * @param entity
     * @return
     */
    @Transactional
    public boolean taking(EmergencyRepair entity) {
        boolean er = updateById(entity);
        boolean re = false;
        if (er) {
            PlatformCompanyLiftRelevance relevance = platformCompanyLiftRelevanceService.findByMtCompanyAndLift(entity.getMtCompanyId(), entity.getLiftId());
            if (null != relevance) {
                //电梯业务状态改成急修中
                relevance.setLiftCompanyStatus(CommonEnum.LiftStatus.EMERGENCY_REPAIR.getCode());
                re = platformCompanyLiftRelevanceService.updateById(relevance);
            }
        }
        if (er && re) {
            return true;
        } else {
            //强制手动事务回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return false;
        }
    }
    /**
     * 根据维保公司和电梯id查找急修
     * @param mtCompanyId
     * @param liftId
     * @return
     */
    public EmergencyRepair findByMtCompanyAndLift(Long mtCompanyId,Long liftId){
        QueryWrapper<EmergencyRepair> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<EmergencyRepair> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(EmergencyRepair::getMtCompanyId, mtCompanyId);
        lambdaQueryWrapper.eq(EmergencyRepair::getLiftId,liftId);
        //只查询一条
        lambdaQueryWrapper.last("LIMIT 1");
        return getOne(lambdaQueryWrapper);
    }

    /**
     * 电梯管理中根据电梯，维保公司和项目查询急修记录
     * @param request
     * @return
     */
    public IPage<RepairResponse> pageByCompanyLiftProject(RepairRequest request){
        long pageSize = request.getPageSize();
        long pageNum = request.getPageNum();
        IPage<RepairResponse> page = new Page<>(pageNum,pageSize);
        return baseMapper.pageByLift(page,request);
    }
    /**
     * 统计维保公司和电梯下是否存在待急修和急修中的急修单
     * @param mtCompanyId
     * @param liftId
     * @return
     */
    public int countRepairing(Long mtCompanyId,Long liftId){
        QueryWrapper<EmergencyRepair> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<EmergencyRepair> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(EmergencyRepair::getMtCompanyId, mtCompanyId);
        lambdaQueryWrapper.eq(EmergencyRepair::getLiftId,liftId);
        lambdaQueryWrapper.in(EmergencyRepair::getStatus,Arrays.asList(Values.Er.STATUS_TO_REPAIRED,Values.Er.STATUS_REPAIRING));
        return count(lambdaQueryWrapper);

    }
    /**
     * 开启维保，暂停急修，修改电梯业务状态为维保中
     * @param mtCompanyId
     * @param liftId
     * @return
     */
    @Transactional
    public boolean startMaintenance(Long mtCompanyId,Long liftId){
        // 如果急修单不存在，直接返回，
        EmergencyRepair emergencyRepair = findByMtCompanyAndLift(mtCompanyId,liftId);
        if(null == emergencyRepair){
            return true;
        }
        // 如果存在，并且急修中
        if(Values.Er.statusRepairing(emergencyRepair.getStatus())){
            //急修改成暂停中
            emergencyRepair.setStatus(Values.Er.STATUS_STOPING);
            boolean er = updateById(emergencyRepair);
            boolean re = false;
            if(er){
                PlatformCompanyLiftRelevance relevance = platformCompanyLiftRelevanceService.findByMtCompanyAndLift(mtCompanyId,liftId);
                if(null != relevance){
                    //电梯业务状态改成维保中
                    relevance.setLiftCompanyStatus(CommonEnum.LiftStatus.MAINTENANCE.getCode());
                    re = platformCompanyLiftRelevanceService.updateById(relevance);
                }
            }
            if(er && re){
                return true;
            }else{
                //强制手动事务回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return false;
            }
        }
        return true;
    }

    /**
     * 停止维保，恢复急修，修改电梯业务状态为急修中
     * @param mtCompanyId
     * @param liftId
     * @return
     */
    public boolean stopMaintenance(Long mtCompanyId,Long liftId){
        //如果急修单不存在，直接返回，
        EmergencyRepair emergencyRepair = findByMtCompanyAndLift(mtCompanyId,liftId);
        if(null == emergencyRepair){
            return true;
        }
        if(Values.Er.statusStoping(emergencyRepair.getStatus())){
            //恢复暂停中的急修为急修中
            emergencyRepair.setStatus(Values.Er.STATUS_REPAIRING);
            boolean er = updateById(emergencyRepair);
            boolean re = false;
            if(er){
                PlatformCompanyLiftRelevance relevance = platformCompanyLiftRelevanceService.findByMtCompanyAndLift(mtCompanyId,liftId);
                if(null == relevance){
                    return false;
                }
                //电梯业务状态改成急修中
                relevance.setLiftCompanyStatus(CommonEnum.LiftStatus.EMERGENCY_REPAIR.getCode());
                re = platformCompanyLiftRelevanceService.updateById(relevance);
            }
            if(er && re){
                return true;
            }else{
                //强制手动事务回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return false;
            }
        }
        return true;
    }

    /**
     * 生成急修单，同时插入急修图片记录
     * @param repair
     * @param erRecordImgs
     * @return
     */
    @Transactional
    public boolean repairOrder(EmergencyRepair repair, List<ErRecordImg> erRecordImgs){
        Judge.notNull(repair);
        Judge.notNull(erRecordImgs);
        boolean re = saveOrUpdate(repair);
        if(re){
            boolean im = erRecordImgService.saveBatch(erRecordImgs);
            if(im){
                return true;
            }else{
                //强制手动事务回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return false;
            }
        }
        return false;
    }

    /**
     * 插入急修收费项，同时更新急修记录总金额
     * @param repair
     * @param erCostItems
     * @return
     */
    @Transactional
    public boolean costAdd(EmergencyRepair repair, List<ErCostItem> erCostItems){
        boolean re = saveOrUpdate(repair);
        if(re){
            boolean er = erCostItemService.saveBatch(erCostItems);
            if(er){
                return true;
            }else{
                //强制手动事务回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return false;
            }
        }
        return false;
    }

    /**
     * 修改收费项，更新急修总金额
     * @param repair
     * @param erCostItems
     * @return
     */
    @Transactional
    public boolean costModify(EmergencyRepair repair,List<ErCostItem> erCostItems){
        boolean re = saveOrUpdate(repair);
        if(re){
            boolean er = erCostItemService.updateBatchById(erCostItems);
            if(er){
                return true;
            }else{
                //强制手动事务回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return false;
            }
        }
        return false;
    }

    /**
     * 删除收费项，修改急修中的总金额
     * @param repair
     * @param erCostItem
     * @return
     */
    @Transactional
    public boolean costDelete(EmergencyRepair repair,ErCostItem erCostItem){
        boolean er = erCostItemService.removeById(erCostItem);
        if(er){
            boolean re = updateById(repair);
            if(re){
                return true;
            }else{
                //强制手动事务回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return false;
            }
        }
        return false;
    }
}
