package cn.com.ty.lift.business.maintenance.service;

import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MaintenanceDetailRequest;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MaintenanceGenerateRequest;
import cn.com.ty.lift.business.project.dao.entity.ProjectLiftRelevance;
import cn.com.ty.lift.business.project.service.ProjectLiftRelevanceService;
import cn.com.ty.lift.business.maintenance.dao.entity.MaintenancePlan;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MaintenancePlanRequest;
import cn.com.ty.lift.business.maintenance.dao.entity.model.response.MaintenancePlanResponse;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MaintenanceModifyRequest;
import cn.com.ty.lift.business.maintenance.dao.mapper.MaintenancePlanMapper;
import cn.com.ty.lift.business.project.dao.entity.Project;
import cn.com.ty.lift.business.project.service.ProjectService;
import cn.com.ty.lift.common.base.ExportRequest;
import cn.com.ty.lift.common.export.ExportUtils;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author bieao
 * @date 2019/12/14
 * @description 维保计划业务层
 */
@Service
@Slf4j
public class MaintenancePlanService extends ServiceImpl<MaintenancePlanMapper, MaintenancePlan> {

    @Resource
    private ProjectService projectService;

    @Resource
    private MaintenancePlanMapper planMapper;

    @Resource
    private ProjectLiftRelevanceService projectLiftRelevanceService;


    private Map<String, String> paramMap = new HashMap<String, String>() {{
        put("projectCode", "项目编号");
        put("projectName", "项目名称");
        put("regionName", "区域名称");
        put("registrationCode", "注册代码");
        put("devicePosition", "电梯位置");
        put("workerName", "维保工");
        put("", "间隔");
        put("liftStatus", "电梯维保状态");
        put("planDate", "计划时间");
    }};

    //保养类型 1.半月，2.季度，3.半年，4.全年
    private int[] maintenanceType = {1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 3, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 4};

    /**
     * @param request 维保计划列表
     * @return 1.成功, 0.失败, 消息描述
     * @description 查询维保计划
     * @date 2019/12/24 10:47 AM
     */
    public IPage<MaintenancePlanResponse> list(MaintenancePlanRequest request) {
        IPage<MaintenancePlanResponse> page = new Page<>(request.getPageNum(), request.getPageSize());
        return planMapper.findByCondition(page, request);
    }

    /**
     * @param request 公司id，项目id，电梯id
     * @return 1.成功, 0.失败, 消息描述
     * @description 查询维保计划详情列表
     * @date 2020/1/2 9:43 上午
     */
    public List<MaintenancePlan> detailList(MaintenanceDetailRequest request) {
        QueryWrapper<MaintenancePlan> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<MaintenancePlan> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(MaintenancePlan::getMtCompanyId, request.getMtCompanyId());
        lambdaQueryWrapper.eq(MaintenancePlan::getProjectId, request.getProjectId());
        lambdaQueryWrapper.eq(MaintenancePlan::getLiftId, request.getLiftId());
        lambdaQueryWrapper.between(MaintenancePlan::getPlanDate, request.getBeginTime(), request.getEndTime());
        return list(lambdaQueryWrapper);
    }

    /**
     * @return 判断结果
     * @description 新增维保计划前置判断条件
     * @date 2020/1/9 3:56 下午
     */
    public boolean preJudgment(List<MaintenancePlan> planList, LocalDate firstTime) {
        Collection<Project> projects = projectService.getProjectList(planList);
        for (Project entry : projects) {
            //若首保时间早于项目开始时间，则不能新增维保计划
            boolean startResult = firstTime.isBefore(entry.getStartDate());
            if (startResult) return false;
            //若首保时间晚于项目结束时间，也不能新增维保计划
            boolean endResult = firstTime.isAfter(entry.getEndDate());
            if (endResult) return false;
        }
        return true;
    }

    /**
     * @param request 电梯列表
     * @return 是否成功
     * @description 批量生成维保计划
     * @date 2019/12/16 2:14 PM
     */
    public RestResponse insertBatch(MaintenanceGenerateRequest request) {
        boolean result = preJudgment(request.getPlanList(), request.getFirstTime());
        if (!result){
            return RestResponse.fail(MessageUtils.get("msg.error.time"));
        }
        List<MaintenancePlan> resultList = addPlan(request);
        boolean ret = saveBatch(resultList, resultList.size());
        if (ret) {
            return RestResponse.success(null, MessageUtils.get("msg.add.success"));
        } else {
            return RestResponse.fail(MessageUtils.get("msg.add.fail"));
        }
    }

    /**
     * @param request 需要保养的电梯
     * @return List<MaintenancePlan> 维保计划列表
     * @description 新增维保计划
     * @date 2019/12/16 1:21 PM
     */
    public List<MaintenancePlan> addPlan(MaintenanceGenerateRequest request) {
        List<MaintenancePlan> plans = request.getPlanList();
        Collection<Project> projects = projectService.getProjectList(request.getPlanList());
        //获取保养间隔
        int interval = request.getInterval();
        //获取从第几期开始
        Integer periods = request.getPeriods();
        //获取首保时间
        LocalDate firstTime = request.getFirstTime();
        List<MaintenancePlan> planList = new ArrayList<>();
        plans.forEach(plan -> {
            Optional<Project> project = projects.stream().filter(p -> p.getId().equals(plan.getProjectId())).findFirst();
            if (project.isPresent()) {
                Project entry = project.get();
                planList.addAll(generatePlan(plan, periods, interval, firstTime, entry.getEndDate()));
            }
        });
        //更新项目电梯关联表中电梯的首保时间和保养间隔
        List<ProjectLiftRelevance> entryList = request.getRelevanceList();
        entryList.forEach(entry -> {
            entry.setPlanInterval(interval);
            entry.setFirstTime(firstTime);
        });
        projectLiftRelevanceService.updateBatchById(entryList);
        return planList;
    }

    /**
     * @param interval 保养间隔
     * @param beginTime 开始时间
     * @param endTime 结束时间
     * @return 期数
     * @description 获取维保计划生成期数
     * @date 2020/1/8 8:11 下午
     */
    public int getTimes(int interval, LocalDate beginTime, LocalDate endTime) {
        //计算当前电梯所在项目开始时间到结束时间需要保养的期数
        long days = beginTime.until(endTime, ChronoUnit.DAYS);
        double num = Math.floor(Double.parseDouble(String.valueOf((days / interval))));
        return (int) num;
    }

    /**
     * @param plan 维保计划
     * @param periods 期数
     * @param interval 保养间隔
     * @param beginTime 开始时间
     * @param endTime 结束时间
     * @return 维保计划列表
     * @description 生成维保计划
     * @date 2020/1/8 8:20 下午
     */
    public List<MaintenancePlan> generatePlan(MaintenancePlan plan, Integer periods, int interval, LocalDate beginTime, LocalDate endTime) {
        //返回维保计划列表
        List<MaintenancePlan> planList = new ArrayList<>();
        int times = getTimes(interval, beginTime, endTime);
        log.info("总期数：{},从第{}期开始,保养间隔：{}天,开始时间：{},结束时间：{}", times + 1, periods, interval, beginTime, endTime);
        int length = maintenanceType.length;
        for (int i = 0; i <= times; i++) {
            MaintenancePlan entry = new MaintenancePlan();
            entry.setWorkerId(plan.getWorkerId());
            entry.setLiftId(plan.getLiftId());
            entry.setProjectId(plan.getProjectId());
            entry.setMtCompanyId(plan.getMtCompanyId());
            int mod = periods % length;
            //设置维保类型
            if (mod == 0) {
                entry.setType(maintenanceType[length - 1]);
            } else {
                entry.setType(maintenanceType[mod - 1]);
            }
            if (i == 0) {
                entry.setPlanDate(beginTime);
            } else {
                //设置维护保养时间
                MaintenancePlan maintenancePlan = planList.get(i - 1);
                LocalDate planDate = maintenancePlan.getPlanDate().plusDays(interval);
                entry.setPlanDate(planDate);
            }
            entry.setCount(periods);
            log.info("第{}期,类型：{},计划时间：{}", periods, entry.getType(), entry.getPlanDate());
            periods++;
            planList.add(entry);
        }
        return planList;
    }

    /**
     * @param request 当前时间
     * @return List<MaintenancePlan> 维保计划列表
     * @description 查询当前时间之后的维保计划
     * @date 2020/1/7 2:40 下午
     */
    public List<MaintenancePlan> findPlanList(MaintenanceModifyRequest request){
        QueryWrapper<MaintenancePlan> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<MaintenancePlan> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(MaintenancePlan::getLiftId, request.getLiftId());
        lambdaQueryWrapper.eq(MaintenancePlan::getProjectId, request.getProjectId());
        lambdaQueryWrapper.eq(MaintenancePlan::getMtCompanyId, request.getMtCompanyId());
        lambdaQueryWrapper.ge(MaintenancePlan::getPlanDate, request.getCurrentTime());
        return list(lambdaQueryWrapper);
    }


    /**
     * @param mtCompanyId 公司id
     * @param liftId 电梯id
     * @return 维保计划列表
     * @description 获取当前期数后的维保计划列表
     * @date 2020/1/8 10:33 上午
     */
    public List<MaintenancePlan> findPlanList(Long mtCompanyId, Long liftId, Integer periods) {
        QueryWrapper<MaintenancePlan> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<MaintenancePlan> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(MaintenancePlan::getMtCompanyId, mtCompanyId);
        lambdaQueryWrapper.eq(MaintenancePlan::getLiftId, liftId);
        lambdaQueryWrapper.ge(MaintenancePlan::getCount, periods);
        return list(lambdaQueryWrapper);
    }

    /**
     * @param request 修改维保计划请求
     * @return 是否修改成功
     * @description 修改维保计划
     * @date 2019/12/16 5:25 PM
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean modifyPlan(MaintenanceModifyRequest request) {
        //获取公司id
        Long mtCompanyId = request.getMtCompanyId();
        //获取电梯id
        Long liftId = request.getLiftId();
        MaintenancePlan plan = new MaintenancePlan();
        plan.setProjectId(request.getProjectId());
        plan.setMtCompanyId(mtCompanyId);
        plan.setLiftId(liftId);
        plan.setWorkerId(request.getWorkerId());
        Project project = projectService.detail(request.getProjectId());
        if (ObjectUtil.isEmpty(project)) return false;
        LocalDate endDate = project.getEndDate();
        //获取当前的保养时间
        LocalDate currentTime = request.getCurrentTime();
        //获取修改的保养时间
        LocalDate updateTime = request.getUpdateTime();
        //获取修改的保养间隔
        Integer updateInterval = request.getInterval();
        //获取当前期数
        Integer periods = request.getPeriods();
        //获取当前电梯的保养间隔
        ProjectLiftRelevance entry = projectLiftRelevanceService.getOne(mtCompanyId, liftId);
        if (ObjectUtil.isEmpty(entry)) return false;
        int interval = entry.getPlanInterval();
        boolean isNotNull = ObjectUtil.isNotEmpty(currentTime) && ObjectUtil.isNotEmpty(updateTime);
        boolean isNull = ObjectUtil.isEmpty(currentTime) && ObjectUtil.isEmpty(updateTime);
        List<MaintenancePlan> planList;
        List<MaintenancePlan> plans = new ArrayList<>();
        if (isNotNull && ObjectUtil.isEmpty(updateInterval)) {
            //获取当前时间后的维保计划列表，并清除计划
            planList = findPlanList(request);
            removeByIds(planList.stream().map(MaintenancePlan::getId).collect(Collectors.toList()));
            //按照修改时间重新生成计划
            plans = generatePlan(plan, periods, interval, updateTime, endDate);
        } else if (isNull && ObjectUtil.isNotEmpty(updateInterval)) {
            //获取当前期数后的维保计划列表，并清除计划
            planList = findPlanList(mtCompanyId, liftId, periods);
            LocalDate planDate;
            Optional<MaintenancePlan> planOptional = planList.stream()
                    .filter(entity -> entity.getCount().equals(periods)).findFirst();
            if (planOptional.isPresent()) {
                MaintenancePlan maintenancePlan = planOptional.get();
                planDate = maintenancePlan.getPlanDate();
            } else return false;
            removeByIds(planList.stream().map(MaintenancePlan::getId).collect(Collectors.toList()));
            //更新项目电梯关联表保养间隔
            projectLiftRelevanceService.modifyPlanInterval(request.getRelevanceId(), updateInterval);
            plans = generatePlan(plan, periods, updateInterval, planDate, endDate);
        } else if (isNotNull && ObjectUtil.isNotEmpty(updateInterval)) {
            //获取当前期数后的维保计划列表，并清除计划
            planList = findPlanList(mtCompanyId, liftId, periods);
            removeByIds(planList.stream().map(MaintenancePlan::getId).collect(Collectors.toList()));
            //更新项目电梯关联表保养间隔
            projectLiftRelevanceService.modifyPlanInterval(request.getRelevanceId(), updateInterval);
            //按照修改时间重新生成计划
            plans = generatePlan(plan, periods, updateInterval, updateTime, endDate);
        }
        return saveBatch(plans, plans.size());
    }

    /**
     * @param mtCompanyId 公司id
     * @param liftList 电梯id列表
     * @return 是否成功
     * @description 批量清除维保计划
     * @date 2019/12/28 2:00 PM
     */
    public boolean removeMaintenancePlan(Long mtCompanyId, Collection<Long> liftList) {
        QueryWrapper<MaintenancePlan> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<MaintenancePlan> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(MaintenancePlan::getMtCompanyId, mtCompanyId);
        lambdaQueryWrapper.in(MaintenancePlan::getLiftId, liftList);
        return remove(queryWrapper);
    }

    /**
     * @param mtCompanyId 公司id
     * @param liftId      电梯id
     * @return 是否成功
     * @description 清空维保计划
     * @date 2019/12/21 3:39 PM
     */
    public boolean cleanPlan(Long mtCompanyId, Long liftId) {
        Map<String, Object> columnMap = new HashMap<>(2);
        columnMap.put("mt_company_id", mtCompanyId);
        columnMap.put("lift_id", liftId);
        return planMapper.deleteByMap(columnMap) > 0;
    }

    /**
     * @param request 导出项目数据列表
     * @return RestResponse 状态码和返回消息
     * @description 导出维保计划
     * @date 2019/12/19 11:27 AM
     */
    public void export(ExportRequest request) {
        List<MaintenancePlan> projectList = planMapper.selectBatchIds(request.getIds());
        ExportUtils utils = new ExportUtils();
        utils.export(projectList, paramMap, request.getExportPath());
    }
}
